<?php

namespace Aedes\EveryPay\Controller\Checkout;

use Aedes\EveryPay\Helper\OrderProcessor;
use Magento\Framework\Controller\ResultFactory;

/**
 * Class Result
 */
class Result extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Aedes\EveryPay\Helper\ResponseHandler
     */
    protected $responseHandler;

    /**
     * @var \Aedes\EveryPay\Logger\Logger
     */
    protected $logger;

    /**
     * @var OrderProcessor
     */
    protected $orderProcessor;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Aedes\EveryPay\Helper\ResponseHandler $responseHandler
     * @param OrderProcessor $orderProcessor
     * @param \Aedes\EveryPay\Logger\Logger $logger
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Aedes\EveryPay\Helper\ResponseHandler $responseHandler,
        \Aedes\EveryPay\Helper\OrderProcessor $orderProcessor,
        \Aedes\EveryPay\Logger\Logger $logger
    ) {
        parent::__construct($context);

        $this->responseHandler = $responseHandler;
        $this->orderProcessor = $orderProcessor;
        $this->logger = $logger;
    }

    /**
     * {@inheritDoc}
     */
    public function execute()
    {
        $response = $this->getRequest()->getParams();
        $result = null;

        try {
            $result = $this->responseHandler->handleResponse($response);
        } catch (\Exception $e) {
            $this->logger->error('Exception while processing response', [$e->getMessage(), $e->getTrace()]);
        }

        if ($this->isPaymentSuccessful($response)) {
            if ($result === true) {
                return $this->_redirect('checkout/onepage/success');
            } else if (is_array($result)) {
                foreach ($result as $error) {
                    $this->messageManager->addErrorMessage($error);
                }

                $this->logger->error('Error while validating response payload', $result);
            }
        } else if (!$this->isPaymentSuccessful($response)) {
            $this->cancelOrderAndRestoreQuote($response);
            return $this->redirectToCart();
        }

        $this->messageManager->addNoticeMessage(
            __('It seems that something went wrong. Please don\'t hesitate to contact us if any errors occurred during the payment process')
        );

        return $this->_redirect('checkout/onepage/failure');
    }

    /**
     * Check whether the payment was successful from the response data
     *
     * @param $response
     * @return bool
     */
    public function isPaymentSuccessful($response)
    {
        $success = false;

        if (isset($response['payment_state'])) {
            if ($response['payment_state'] == 'settled' || $response['payment_state'] == 'authorized') {
                $success = true;
            } else if ($response['payment_state'] == 'cancelled'
                || $response['payment_state'] == 'waiting_for_3ds_response'
                || $response['payment_state'] == 'failed') {
                $success =  false;
            }
        } else if (isset($response['cancel'])) { //from IFrame
            $success =  false;
        }

        return $success;
    }

    /**
     * Cancel the order, add a notice
     *
     * @param $response
     * @return void
     */
    public function cancelOrderAndRestoreQuote($response)
    {
        try {
            $this->orderProcessor->cancelOrderAndRestoreQuote();
            $this->messageManager->addNoticeMessage(__('Order has been cancelled.'));
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            $this->messageManager->addExceptionMessage($e, $e->getMessage());
            $this->logger->error('Error while cancelling the order', [$response, $e->getMessage(), $e->getTrace()]);
        } catch (\Exception $e) {
            $this->messageManager->addExceptionMessage($e, __('Unable to cancel order'));
            $this->logger->error('Error while cancelling the order', [$response, $e->getMessage(), $e->getTrace()]);
        }
    }

    /**
     * Redirect to cart page
     *
     * @return \Magento\Framework\Controller\Result\Redirect
     */
    public function redirectToCart() {
        /** @var \Magento\Framework\Controller\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        return $resultRedirect->setPath('checkout/cart');
    }
}