<?php

namespace Aedes\EveryPay\Helper;

class Validator extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * This is a minor security flaw, but this check is needed to bypass redirection to failure page
     * if EveryPay sent the same nonce back twice. This situation occurs when the customer
     * clicks on the 'back' button when EveryPay had already sent data back to merchant.
     */
    const NONCE_AGE = 60;

    /**
     * Taken from https://github.com/UnifiedPaymentSolutions/everypay-integration/blob/master/php/EveryPay.php::157
     * (as of 25.01.2017)
     */
    const RESPONSE_AGE = 600;

    /**
     * @var \Magento\Sales\Api\Data\OrderInterface
     */
    protected $orderInterface;

    /**
     * @var \Aedes\EveryPay\Model\Config
     */
    protected $config;

    /**
     * @var \Aedes\EveryPay\Model\Hmac\HmacManagement
     */
    protected $hmac;

    /**
     * @var \Aedes\EveryPay\Model\Nonce\NonceManagement
     */
    protected $nonce;

    /**
     * @var array
     */
    protected $errors = [];

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Sales\Api\Data\OrderInterface $orderInterface
     * @param \Aedes\EveryPay\Model\Config $config
     * @param \Aedes\EveryPay\Model\Hmac\HmacManagement $hmac
     * @param \Aedes\EveryPay\Model\Nonce\NonceManagement $nonce
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Sales\Api\Data\OrderInterface $orderInterface,
        \Aedes\EveryPay\Model\Config $config,
        \Aedes\EveryPay\Model\Hmac\HmacManagement $hmac,
        \Aedes\EveryPay\Model\Nonce\NonceManagement $nonce
    ) {
        $this->orderInterface = $orderInterface;
        $this->config = $config;
        $this->hmac = $hmac;
        $this->nonce = $nonce;

        parent::__construct($context);
    }

    /**
     * @param array $data
     * @return bool|array
     */
    public function validate(array $data)
    {
        $this->errors = [];

        $this
            ->validateHmac($data)
            ->validateTimestamp($data)
            ->validateNonce($data)
            ->validateOrderReference($data)
            ->validateUsername($data);

        if (!empty($this->errors)) {
            return $this->errors;
        }

        return true;
    }

    /**
     * @param array $data
     * @return $this
     */
    protected function validateUsername(array $data)
    {
        if (!isset($data['api_username']) || $data['api_username'] !== $this->config->getApiUser()) {
            $this->errors[] = __('Response username is invalid');
        }

        return $this;
    }

    /**
     * @param array $data
     * @return $this
     */
    protected function validateOrderReference(array $data)
    {
        try {
            $this->orderInterface->loadByIncrementId($data['order_reference']);
        } catch (\Exception $e) {
            $this->errors[] = __('Order does not exist');
        }

        return $this;
    }

    /**
     * @param array $data
     * @return $this
     */
    protected function validateNonce(array $data)
    {
        if (!isset($data['nonce']) || !$this->nonce->isNonceUnique($data['nonce'], self::NONCE_AGE)) {
            $this->errors[] = __('Response nonce is invalid');
        }

        return $this;
    }

    /**
     * @param array $data
     * @return $this
     */
    protected function validateTimestamp(array $data)
    {
        $now = time();

        if (!isset($data['timestamp']) ||
            ($data['timestamp'] > $now) || ($data['timestamp'] < ($now - self::RESPONSE_AGE))
        ) {
            $this->errors[] = __('Response is outdated');
        }

        return $this;
    }

    /**
     * @param array $data
     * @return $this
     */
    protected function validateHmac(array $data)
    {
        if (!isset($data['hmac_fields']) || !$this->hmac->isResponseHmacValid($data)) {
            $this->errors[] = __('Response HMAC is invalid');
        }

        return $this;
    }
}